--[[---------------------------------------------------------------------------
	
	Creator: x[N]ir - #4841
	This work is licensed under the Creative Commons 
	Attribution-NonCommercial-ShareAlike 4.0 International License. 
	License details: http://creativecommons.org/licenses/by-nc-sa/4.0/
	
-----------------------------------------------------------------------------]]

--[[---------------------------------------------------------------------------
	Adds a new camera

	@param table pos: coordinates in pixels
	@param number type: camera's type (idle / mobile)
	@param number rot: camera's starting rotation (optionnal)
	@param number laserRange: how far the laser can go in tiles (optionnal)
	@param table laserColor: the laser color {R,G,B} (optionnal)
	@param number hp: camera's hp (optionnal)
	@param number rotDelay: How much seconds the camera takes to rotate (only
		for mobile type)
	@param number endRot: final angle for mobile cameras
-----------------------------------------------------------------------------]]
function camera.add( pos, type, rot, laserRange, laserColor, hp, rotDelay, endRot )
	camera.list[ #camera.list + 1 ] = {
		-- Params
		x = pos[ 1 ],
		y = pos[ 2 ],
		type = type,
		
		-- Optionnal Params
		rot = rot or 0,
		laserRange = ( laserRange or camera.DEFAULT_LASER_RANGE ),
		laserColor = laserColor or camera.DEFAULT_LASER_COLOR,
		hp = hp or camera.DEFAULT_HP,
		baseHp = hp or camera.DEFAULT_HP,
		rotDelay = rotDelay or camera.DEFAULT_ROT_DELAY,
		endRot = endRot or ( rot + 90 ),

		-- Default Properties
		id = #camera.list + 1,
		images = {},
		status = camera.OK,
		towardsEndRot = false,
	}
end

--[[---------------------------------------------------------------------------
	Removes the specified camera from the map only !

	@param number id: the camera's ID
-----------------------------------------------------------------------------]]
function camera.remove( id )
	local cam = camera.list[ id ]

	-- Removing all images
	for _, imageID in pairs( cam.images ) do
		freeimage( imageID )
	end

	cam.images = {}
end

--[[---------------------------------------------------------------------------
	Destroys the specified camera

	@param Camera cam: a camera
-----------------------------------------------------------------------------]]
function camera.destroy( cam )
	-- Remove all images
	camera.remove( cam.id )

	-- Make an Explosion
	parse('explosion '..cam.x..' '..cam.y..' 64 30 0')

	-- Remove the timer in case of rotating
	freetimer( "parse", 'lua camera.rotate('..cam.id..')' )
end

--[[---------------------------------------------------------------------------
	Displays the specified camera in the map

	@param number id: a camera ID
-----------------------------------------------------------------------------]]
function camera.display( id )
	local cam = camera.list[ id ]

	-- Is this camera already displayed ?
	if cam.images[ 1 ] then return end

	-- Images
	local camImageID = image( camera.GFX_CAMERA, 0, 0, 1 )
	local laserImageID = image( camera.GFX_LASER, 0, 0, 1 )

	-- Position + Rotation
	imagepos( camImageID, cam.x, cam.y, cam.rot )
	imagepos( laserImageID, cam.x, cam.y, cam.rot )

	-- Laser Color & Scale
	imagecolor( laserImageID, cam.laserColor[ 1 ], cam.laserColor[ 2 ],
		cam.laserColor[ 3 ] )
	imagescale( laserImageID, cam.laserRange * 32, 1)

	-- Hitzone
	imagehitzone(camImageID, 101, 
		-camera.CAMERA_WIDTH * 0.5, -camera.CAMERA_HEIGHT * 0.5,
		camera.CAMERA_WIDTH, camera.CAMERA_HEIGHT)

	-- Saving vars
	cam.images[ 1 ] = camImageID
	cam.images[ 2 ] = laserImageID
	cam.hp = cam.baseHp

	-- Mobile Effect
	if cam.type == camera.MOBILE then
		cam.towardsEndRot = true

		-- Rotation Effect for both camera and laser
		tween_rotate( cam.images[ 1 ], cam.rotDelay * 1000, cam.endRot )
		tween_rotate( cam.images[ 2 ], cam.rotDelay * 1000, cam.endRot )

		-- Infinite timer for rotation
		timer( cam.rotDelay * 1000, "parse", 'lua camera.rotate('..id..')' )
	end
end

--[[---------------------------------------------------------------------------
	Rotates from an angle to an other one

	@param number id: a camera id
-----------------------------------------------------------------------------]]
function camera.rotate( id )
	-- The camera
	local cam = camera.list[ id ]
	-- Current cam angle
	local angle = cam.rot

	-- Update the state
	if not cam.towardsEndRot then
		angle = cam.endRot
		cam.towardsEndRot = true
	else
		cam.towardsEndRot = false
	end

	-- Rotation Effect for both camera and laser
	tween_rotate( cam.images[ 1 ], cam.rotDelay * 1000, angle )
	tween_rotate( cam.images[ 2 ], cam.rotDelay * 1000, angle )

	-- Then the camera will go back
	timer( cam.rotDelay * 1000, "parse", 'lua camera.rotate('..id..')' )
end

--[[---------------------------------------------------------------------------
	Check a player position and triggers alarm(s) if necessary
	
	@param number id: a player ID
	@param number playerX: player x tile position
	@param number playerY: player y tile position
-----------------------------------------------------------------------------]]
function camera.isDetected( id, playerX, playerY )
	-- Through all cameras
	for camID, cam in pairs( camera.list ) do
		-- Base Image
		local camImageID = cam.images[ 1 ]

		-- The camera should be displayed and in standard mode
		if cam.status == camera.OK and camImageID then 
			-- Fist we get the camera angle.
			-- Substracted by 90 because the top of the camera
			-- goes to the left in the default image
			local camAngle = object( camImageID, "rot" ) - 90

			-- Then from this angle,
			-- We check every tile from the camera pos to the laser range
			for i = 1, cam.laserRange do
				-- Next position in pixels
				local cx = cam.x + math.sin( math.rad( camAngle ) ) * ( i * 32 )
				local cy = cam.y - math.cos( math.rad( camAngle ) ) * ( i * 32 )
				-- Converted to tiles
				local laserX = math.floor( cx / 32 )
				local laserY = math.floor( cy / 32 )

				-- If the player is inside one of these tiles
				-- The alarm is triggered 
				if playerX == laserX and playerY == laserY then
					-- Update the status
					cam.status = camera.ALARM
					-- Repeat the sound
					parse( 'sv_sound ' .. camera.SFX_ALARM )
					timer( 1000, "parse", 'sv_sound ' .. camera.SFX_ALARM,
						( camera.ALARM_DELAY - 1 ) )
					-- Switch the state back to normal
					timer( 1000 * ( camera.ALARM_DELAY - 1 ),
						"parse", "lua camera.list[" .. camID .. "].status=camera.OK" )
				end
			end
		end
	end
end

--[[---------------------------------------------------------------------------
	Returns a camera by it's image id or nil if not found

	@param number imageID: a camera image ID
-----------------------------------------------------------------------------]]
function camera.getCameraByImageID( imageID )
	-- Through all cameras
	for __, cam in pairs( camera.list ) do
		-- Base Image ID is the same as the parameter
		if cam.images[ 1 ] == imageID then
			return cam
		end
	end

	-- Not found
	return nil
end